from sqlalchemy.orm import relationship
from sqlalchemy import Column, Integer, String, ForeignKey, Enum

from arepo.base import Base

change_type = Enum('addition', 'deletion', name='change_type')


class ChangeModel(Base):
    __tablename__ = 'change'
    id = Column(String, primary_key=True)
    line = Column('number', Integer, nullable=False)
    content = Column('content', String, nullable=False)
    start_col = Column(Integer)
    end_col = Column(Integer)
    type = Column('type', change_type)
    diff_block_id = Column(String, ForeignKey('diff_block.id'))


class DiffBlockModel(Base):
    __tablename__ = 'diff_block'
    id = Column(String, primary_key=True)
    order = Column(Integer)
    changes = relationship("ChangeModel", backref="diff_block")
    a_path = Column('a_path', String, nullable=False)
    commit_file_id = Column(String, ForeignKey('commit_file.id'))



from arepo.base import Base

from sqlalchemy import Column, String, ForeignKey, Boolean, Float, ForeignKeyConstraint


class CVSS2Model(Base):
    __tablename__ = "cvss2"

    id = Column(String, primary_key=True)
    vector_string = Column('vector_string', String)
    access_vector = Column('access_vector', String)
    access_complexity = Column('access_complexity', String)
    authentication = Column('authentication', String)
    confidentiality_impact = Column('confidentiality_impact', String)
    integrity_impact = Column('integrity_impact', String)
    availability_impact = Column('availability_impact', String)
    base_severity = Column('base_severity', String, nullable=True)
    base_score = Column('base_score', Float, nullable=True)
    exploitability_score = Column('exploitability_score', Float, nullable=True)
    impact_score = Column('impact_score', Float, nullable=True)
    # Additional attributes
    ac_insuf_info = Column('ac_insuf_info', Boolean, nullable=True)
    obtain_all_privilege = Column('obtain_all_privilege', Boolean, nullable=True)
    obtain_user_privilege = Column('obtain_user_privilege', Boolean, nullable=True)
    obtain_other_privilege = Column('obtain_other_privilege', Boolean, nullable=True)
    user_interaction_required = Column('user_interaction_required', Boolean, nullable=True)
    # type = Column('type', String, nullable=True)  # TODO: to be added later
    # version = Column('version', String, nullable=True)  # TODO: put back if needed


# TODO: there must be a better way to handle this
class CVSS2AssociationModel(Base):
    __tablename__ = 'cvss2_association'
    __table_args__ = (
        ForeignKeyConstraint(('cvss_id',), ['cvss2.id']),
        ForeignKeyConstraint(('source_id',), ['source.id']),
        ForeignKeyConstraint(('vulnerability_id',), ['vulnerability.id'])
    )

    cvss_id = Column(String, ForeignKey('cvss2.id'), primary_key=True)
    source_id = Column(String, ForeignKey('source.id'), primary_key=True)
    vulnerability_id = Column(String, ForeignKey('vulnerability.id'), primary_key=True)



from arepo.base import Base
from sqlalchemy import Column, String, ForeignKey, Float, ForeignKeyConstraint


class VulnerabilityCVSS3(Base):
    __tablename__ = 'vulnerability_cvss3'
    __table_args__ = (
        ForeignKeyConstraint(('cvss_id',), ['cvss3.id']),
        ForeignKeyConstraint(('vulnerability_id',), ['vulnerability.id'])
    )

    cvss_id = Column(String, ForeignKey('cvss2.id'), primary_key=True)
    vulnerability_id = Column(String, ForeignKey('vulnerability.id'), primary_key=True)


class CVSS3Model(Base):
    __tablename__ = "cvss3"

    id = Column(String, primary_key=True)
    version = Column('version', String)
    vector_string = Column('vector_string', String)
    attack_vector = Column('attack_vector', String)
    attack_complexity = Column('attack_complexity', String)
    privileges_required = Column('privileges_required', String)
    user_interaction = Column('user_interaction', String)
    scope = Column('scope', String)
    confidentiality_impact = Column('confidentiality_impact', String)
    integrity_impact = Column('integrity_impact', String)
    availability_impact = Column('availability_impact', String)
    base_severity = Column('base_severity', String, nullable=True)
    base_score = Column('base_score', Float, nullable=True)
    exploitability_score = Column('exploitability_score', Float, nullable=True)
    impact_score = Column('impact_score', Float, nullable=True)
    # type = Column('type', String, nullable=True)  # TODO: to be added later


# TODO: there must be a better way to handle this
class CVSS3AssociationModel(Base):
    __tablename__ = 'cvss3_association'
    __table_args__ = (
        ForeignKeyConstraint(('cvss_id',), ['cvss3.id']),
        ForeignKeyConstraint(('source_id',), ['source.id']),
        ForeignKeyConstraint(('vulnerability_id',), ['vulnerability.id'])
    )

    cvss_id = Column(String, ForeignKey('cvss3.id'), primary_key=True)
    source_id = Column(String, ForeignKey('source.id'), primary_key=True)
    vulnerability_id = Column(String, ForeignKey('vulnerability.id'), primary_key=True)



from arepo.base import Base
from sqlalchemy import Column, String, ForeignKey, Integer, Boolean
from sqlalchemy.orm import relationship


class CPEModel(Base):
    __tablename__ = "cpe"

    id = Column('id', String, primary_key=True)
    product_id = Column(String, ForeignKey('product.id'), nullable=False)
    version = Column(String, nullable=True, default="*")
    update = Column(String, nullable=True, default="*")
    edition = Column(String, nullable=True, default="*")
    language = Column(String, nullable=True, default="*")
    sw_edition = Column(String, nullable=True, default="*")
    target_sw = Column(String, nullable=True, default="*")
    target_hw = Column(String, nullable=True, default="*")
    other = Column(String, nullable=True, default="*")

    product = relationship("ProductModel", back_populates="cpes")
    cpe_matches = relationship("CPEMatchModel", back_populates="cpe")


class CPEMatchModel(Base):
    __tablename__ = "cpe_match"

    id = Column('id', String, primary_key=True)
    cpe_id = Column(String, ForeignKey('cpe.id'), nullable=False)
    node_id = Column(String, ForeignKey('node.id'), nullable=False)
    vulnerable = Column(Boolean, nullable=False)
    version_start_including = Column(String, nullable=True)
    version_start_excluding = Column(String, nullable=True)
    version_end_including = Column(String, nullable=True)
    version_end_excluding = Column(String, nullable=True)

    cpe = relationship("CPEModel", back_populates="cpe_matches")
    node = relationship("NodeModel", back_populates="cpe_matches")



import pandas as pd
from pathlib import Path

from sqlalchemy.orm import relationship
from sqlalchemy import Column, Integer, String, ForeignKey, PrimaryKeyConstraint

from arepo.base import Base


class LabelModel(Base):
    __tablename__ = "label"

    id = Column('id', Integer, primary_key=True)
    name = Column('name', String, nullable=False)


class FunctionModel(Base):
    __tablename__ = "function"

    id = Column('id', String, primary_key=True)
    name = Column('name', String, nullable=False)
    commit_file_id = Column(String, ForeignKey('commit_file.id'))
    start_line = Column('start_line', Integer, nullable=False)
    start_col = Column('start_col', Integer, nullable=False)
    end_line = Column('end_line', Integer, nullable=False)
    end_col = Column('end_col', Integer, nullable=False)
    size = Column('size', Integer, nullable=False)
    content = Column('content', String, nullable=False)


class TopicModel(Base):
    __tablename__ = "topic"

    id = Column('id', String, primary_key=True)
    name = Column('name', String, nullable=False)
    repositories = relationship("RepositoryModel", secondary="repository_topic", backref='topics')

    @staticmethod
    def populate(tables_path: Path):
        topics_df = pd.read_csv(f'{tables_path}/topics.csv')
        Base.metadata.bind.execute(TopicModel.__table__.insert(), topics_df.to_dict(orient="records"))


class RepositoryTopicModel(Base):
    __tablename__ = 'repository_topic'
    __table_args__ = (
        PrimaryKeyConstraint('repository_id', 'topic_id'),
    )

    repository_id = Column('repository_id', String, ForeignKey('repository.id'))
    topic_id = Column('topic_id', String, ForeignKey('topic.id'))
